package mdd.client;


import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import org.json.simple.JSONObject;

import java.util.Vector;


public class Module extends Group implements IGUIEventClient{
    private boolean _dragged = false;
    private Point2D _pos = new Point2D(0.0f,0.0f);
    private GUIEventHandler _eventHandler = new GUIEventHandler();

    public String ID = "";
    public HBox hlayout = new HBox();
    public VBox inputs = new VBox();
    public VBox outputs = new VBox();
    public boolean isSelected = false;

    public Module(String id){
        ID = id;
        Rectangle titlebar;
        Rectangle background;
        Text caption;
        addInput(new Input(this,"1"));
        addInput(new Input(this,"2"));
        addInput(new Input(this,"3"));

        addOutput(new Output(this,"1"));
        addOutput(new Output(this,"2"));
        addOutput(new Output(this,"3"));

        inputs.setSpacing(10);

        outputs.setSpacing(10);


        hlayout.getChildren().addAll(inputs, outputs);
        hlayout.setSpacing(20);
        hlayout.setTranslateX(-7);
        hlayout.setTranslateY(30);

        //System.out.println("[Module]: pressed: "+ hbox.getPrefWidth());

        //JSONObject parameters
        titlebar = new Rectangle(0.0f, 0.0f, 150-20, 20.0f);
        titlebar.setArcWidth(10.0f);
        titlebar.setArcHeight(10.0f);
        titlebar.setFill(Color.ROYALBLUE);
        background = new Rectangle(0.0f, 0.0f, 150-20, 150 + 40);
        background.setArcWidth(10.0f);
        background.setArcHeight(10.0f);
        background.setFill(Color.GRAY);
        caption = new Text("TEST");
        caption.setFont(new Font("Arial", 15));
        caption.setFill(Color.WHITE);
        caption.setX(10);
        caption.setY(15);


        getChildren().addAll( background, titlebar, caption, hlayout);



        addEventHandler(MouseEvent.MOUSE_PRESSED, e -> {
            System.out.println("[Module]: press event filter detected!");
            if(e.getButton() == MouseButton.PRIMARY) {
                isSelected = true;
                System.out.println("[Module]: select: " + ID);
                JSONObject json = new JSONObject();
                json.put("id", id);
                _eventHandler.publishEvent("MOUSE_PRESSED", json);
                _pos = new Point2D(e.getSceneX(), e.getSceneY());
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
            e.consume();
        });

        addEventHandler(MouseEvent.MOUSE_DRAGGED, e -> {
            //System.out.println("[Module]: dragg event detected!");
            if(e.getButton() == MouseButton.PRIMARY) {
                setTranslateX(getTranslateX()+(e.getSceneX()-_pos.getX()));
                setTranslateY(getTranslateY()+(e.getSceneY()-_pos.getY()));
                _pos = new Point2D( e.getSceneX(),  e.getSceneY());
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
        });
    }

    public void addInput(Input in){
        in.setGUIEventHandler(_eventHandler);
        inputs.getChildren().add(in);
    }

    public void addOutput(Output out){
        out.setGUIEventHandler(_eventHandler);
        outputs.getChildren().add(out);
    }

    public Input getInput(String id){
        for (Node node : inputs.getChildren()){
            if(node instanceof Input){
                Input in = (Input) node;
                if (in.ID.equals(id)){
                    return in;
                }
            }

        }
        System.out.println("[Module]: getInput: WRONG ID!");
        return new Input(this,"");
    }

    public Output getOutput(String id){
        for (Node node : outputs.getChildren()){
            if(node instanceof Output){
                Output out = (Output) node;
                if (out.ID.equals(id)){
                    return out;
                }
            }

        }
        System.out.println("[Module]: getOutput: WRONG ID!");
        return new Output(this,"");
    }

    public GUIEventHandler getGUIEventHandler(){
        return _eventHandler;
    }

    public void setGUIEventHandler(GUIEventHandler eventHandler){
        _eventHandler = eventHandler;
        for (Node node : inputs.getChildren()){
            if(node instanceof Input){
                Input in = (Input) node;
                in.setGUIEventHandler(_eventHandler);
            }

        }

        for (Node node : outputs.getChildren()){
            if(node instanceof Output){
                Output out = (Output) node;
                out.setGUIEventHandler(_eventHandler);
            }

        }
        _eventHandler.addEventListener(this);
    }

    @Override
    public void processGUIEvent(String event, JSONObject args) {
        if (event == "MOUSE_PRESSED"){
            JSONObject json = new JSONObject();
            json.put("id", ID);

            if (!args.equals(json)){
                if (isSelected){
                    System.out.println("[Module]: deselect: " + ID);
                }
                isSelected = false;
            }
        }
    }
}
