package mdd.client;

import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.*;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

public class Processor extends Module{
    private Group modules = new Group();
    private Group connections = new Group();
    public Group all = new Group(connections, modules);
    public Module selected = new Module("-1");
    private GUIEventHandler _eventHandler = new GUIEventHandler();

    public Processor(String id) {
        super( id);
        //setFocusTraversable(true);
        setOnKeyPressed(e -> {
            System.out.println("[Processor]: pressed:");
            System.out.println("[Processor]: pressed: " + e.getText());
            if (isSelected) {
                JSONObject json = new JSONObject();
                json.put("id", ID);
                _eventHandler.publishEvent("OPEN_PROCESSOR", json);
            }
        });

        modules.addEventHandler(DragEvent.DRAG_DROPPED, e -> {
            Dragboard db = e.getDragboard();
            boolean success = false;
            if (db.hasString()) {
                JSONParser parser = new JSONParser();
                JSONArray jsonList = null;
                try {
                    jsonList = (JSONArray)parser.parse(db.getString());
                } catch (ParseException parseException) {
                    parseException.printStackTrace();
                }

                assert jsonList != null;
                if (jsonList.size() == 2){

                    boolean foundInput = false;
                    boolean foundOutput = false;
                    Input in = new Input(this, "-1");
                    Output out = new Output(this, "-1");
                    for (Object o : jsonList) {
                        JSONObject json = (JSONObject) o;
                        if (json.containsKey("type")) {
                            String type = json.get("type").toString();
                            if (type.equals("INPUT")) {
                                System.out.println("[WorkBench] Dropped found type = " + "INPUT");
                                foundInput = true;
                                JSONArray idList = (JSONArray) json.get("ID");
                                in = getModule(idList.get(1).toString()).getInput(idList.get(0).toString());
                            } else if (type.equals("OUTPUT")) {
                                System.out.println("[WorkBench] Dropped found type = " + "OUTPUT");
                                foundOutput = true;
                                JSONArray idList = (JSONArray) json.get("ID");
                                out = getModule(idList.get(1).toString()).getOutput(idList.get(0).toString());
                            }
                        }
                    }
                    if (foundInput && foundOutput){
                        System.out.println("[WorkBench] Dropped: success!");
                        connections.getChildren().add(new Connection(in, out));
                        success = true;
                    }

                }

                System.out.println("[WorkBench] Dropped: " + db.getString());

            }
            e.setDropCompleted(success);
            e.consume();
        });
    }

    public Module getModule(String id){
        for (Node node : modules.getChildren()){
            if (node instanceof Module){
                Module m = (Module)node;
                if (m.ID.equals(id)){
                    return m;
                }
            }
        }
        return new  Module("");
    }

    public void setGUIEventHandler(GUIEventHandler eventHandler){
        _eventHandler = eventHandler;
        for (Node node : inputs.getChildren()){
            if(node instanceof Input){
                Input in = (Input) node;
                in.setGUIEventHandler(_eventHandler);
            }

        }

        for (Node node : outputs.getChildren()){
            if(node instanceof Output){
                Output out = (Output) node;
                out.setGUIEventHandler(_eventHandler);
            }

        }
        _eventHandler.addEventListener(this);
    }

    public void addModule(Module module){
        module.setGUIEventHandler(getGUIEventHandler());
        modules.getChildren().add(module);
    }

    public void addConnection(Connection connection){
        connection.setGUIEventHandler(getGUIEventHandler());
        connections.getChildren().add(connection);
    }
}
